﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net.Sockets;
using System.Net;
using System.IO;

//
// Demonstrates how to interact with Nimbus using the Generic TCP (Server) protocol
//
// This code lacks error handling and will not win any design prize...
//
// Date       / Sign / Vers     / Comment
// ---------------------------------------------------------------------------------------------------
// 2016.11.04 /  TR  / 1.0.0.0  / Main
// 2017.10.13 /  TR  / 1.0.0.1  / Added the message 200 to update receiver info
// 2020.09.02 /  TR  / 1.0.0.2  / Added text file append stuff
//
//
// ---------------------------------------------------------------------------------------------------
//
//
//

namespace NimbusAlarmClient
{

	public partial class NimbusAlarmClient_Main : Form
	{

		private TcpClient nimbusSocket = new TcpClient();

		private NetworkStream nimbusStream;

		public NimbusAlarmClient_Main()
		{
			InitializeComponent();
		}

		private void button_Connect_Click(object sender, EventArgs e)
		{

			nimbusSocket.Connect(textBox_IPAddress.Text, int.Parse(textBox_IPPort.Text));
			nimbusStream = nimbusSocket.GetStream();

		}

		private void button_Disconnect_Click(object sender, EventArgs e)
		{

			nimbusStream.Close();
			nimbusSocket.Close();

		}

    private string CreateAlarmRow()
    {

      string sAlarmRow = "";

			if (radioButton_InActive.Checked)
			{
				sAlarmRow = "0";
			}
			else
			{
        if (radioButton_Active.Checked)
        {
          sAlarmRow = "1";
        }
        else
        {
          if (radioButton_Acked.Checked)
          {
            sAlarmRow = "2";
          }
          else
          {
            return null;
          }
        }
			}

			//
			// STATUS_INACTIVE = 0
			// STATUS_ACTIVE   = 1
			// STATUS_ACKED    = 2
			// STATUS_MESSAGE  = 3
			//
			// MsgType|   Date   |  Time  |    Tag    |Area|Category|      Name    |    Desc    |State
			//   0-2  |2002-12-28|23:56:01|TA0220'GT41| A5 |    2   |Pressure Fault|Pressure bad|ACTIVE
			//
			//
			// Date and time are not needed, it will automatically be set to 'now' by the Nimbus Server
			//

			sAlarmRow += "|" + textBox_Date.Text + "|" + textBox_Time.Text + "|" + textBox_T0.Text + "|" + textBox_T1.Text + "|" + textBox_T2.Text + "|" + textBox_T3.Text + "|" + textBox_T4.Text + "|" + textBox_T5.Text;

      return sAlarmRow;

    }

		private void button_SendAlarm_Click(object sender, EventArgs e)
		{

      Encoding encoding = Encoding.GetEncoding("Windows-1252");

      if (nimbusSocket.Connected == false)
      {
        return;
      }

      string sAlarmRow = CreateAlarmRow();

      if (sAlarmRow == null || sAlarmRow == "")
      {
        return;
      }

      // End each alarm with a LF
      byte[] byteStream = encoding.GetBytes(sAlarmRow + "\n");

			nimbusStream.Write(byteStream, 0, byteStream.Length);
			nimbusStream.Flush();

		}


    private void button_WriteEventToFile_Click(object sender, EventArgs e)
    {

      string sAlarmRow = CreateAlarmRow();

      if (sAlarmRow == null || sAlarmRow == "")
      {
        return;
      }

      try
      {
        // End each alarm with a CR/LF
        // Needs iso-8859-1 encoding
        // Nimbus will first try to rename the file then import it before erasing the file
        // File can't be renamed if it is opened during the write sequence == fail safe import algorithm
        File.AppendAllText(textBox_FileName.Text, sAlarmRow + "\r\n", System.Text.Encoding.GetEncoding("iso-8859-1"));
      }
      catch
      {
      }

    }

    private void NimbusAlarmClient_Main_Load(object sender, EventArgs e)
		{

      string AssemblyName = System.Reflection.Assembly.GetExecutingAssembly().Location;
      System.IO.FileInfo file = new System.IO.FileInfo(AssemblyName);
      string DirectoryName = file.Directory.FullName;

      textBox_FileName.Text = DirectoryName + "\\" + textBox_FileName.Text;

      timer_DateAndTime.Enabled = true;

		}

		private void NimbusAlarmClient_Main_FormClosing(object sender, FormClosingEventArgs e)
		{

			timer_DateAndTime.Enabled = false;

		}

		private void timer_DateAndTime_Tick(object sender, EventArgs e)
		{

			if (checkBox_SetDateAndTime.Checked)
			{
				textBox_Date.Text = DateTime.Now.ToString("yyyy-MM-dd");
				textBox_Time.Text = DateTime.Now.ToString("HH:mm:ss");
			}

			button_SendEvent.Enabled = nimbusSocket.Connected;
			button_SendMessage.Enabled = nimbusSocket.Connected;
			button_GetAllReceivers.Enabled = nimbusSocket.Connected;
			button_UpdateReceiver.Enabled = nimbusSocket.Connected; 
		}

		private void button_SendMessage_Click(object sender, EventArgs e)
		{

			Encoding encoding = Encoding.GetEncoding("Windows-1252");

			string sMessageRow = "";

			if (nimbusSocket.Connected == false)
			{
				return;
			}
 
			//
      // MsgType|   Date   |  Time  |    Tag     |Area|Category|    Name    | Desc  |State
      //    3   |2002-12-28|23:56:01|    n/a     | n/a|   n/a  |ReceiverName|Message| n/a
      //

			//
			// Date and time are not needed, it will automatically be set to 'now' by the Nimbus Server
			//
			//
			// The text '\r\n' in the stream will be exchanged to CR/LF by the Nimbus Server before sending takes place
			//
			// You need Nimbus version 3 version 3.00.08ß4 or later (previous versions had a bug causing messages to never be queued)
			//
			// Nimbus 1 and 2 would also send messages using this method
			//

			sMessageRow = "3||||||" + comboBox_Receivers.Text + "|" + textBox_Message.Text.Replace("\r\n", "\\r\\n") + "\n";

			byte[] byteStream = encoding.GetBytes(sMessageRow);

			nimbusStream.Write(byteStream, 0, byteStream.Length);
			nimbusStream.Flush();

		}

		private void button_GetAllReceivers_Click(object sender, EventArgs e)
		{

			Encoding encoding = Encoding.GetEncoding("Windows-1252");

			if (nimbusSocket.Connected == false)
			{
				return;
			}

			byte[] byteStream = encoding.GetBytes("100" + "\n");

			nimbusStream.Write(byteStream, 0, byteStream.Length);
			nimbusStream.Flush();

			string sReadBuffer = "";
			byte[] byteReadBuffer = new byte[2048];
			int iReadBytes = 0;

			// Not very nice code...don't copy it. This should actually be in a separate thread.
			try
			{
				for (int iTimeout = 0; iTimeout < 500; iTimeout += 100)
				{
					if (nimbusStream.DataAvailable)
					{
						iReadBytes = nimbusStream.Read(byteReadBuffer, 0, byteReadBuffer.Length);
						sReadBuffer += encoding.GetString(byteReadBuffer, 0, iReadBytes);
					}
					System.Threading.Thread.Sleep(100);
				}
			}
			catch
			{
			}

			//
			// Nimbus returns all receivers separated by LF, when the last receiver is sent an extra LF will be sent,
			// hence this split will always have an empty sRecever[] as last element
			//
			// The receivers are sent with ReceiverName followed by the 'enabled' property (checked or not in the Receiver form), receiver type id
			// and the four parameters (phone number etc), all fields separated by a pipe char '|'
			//

			string[] sReceivers = sReadBuffer.Split('\n');

			if (sReceivers.GetLength(0) > 0)
			{

				//MessageBox.Show("Found " + (sReceivers.GetLength(0) - 1).ToString() + " receivers, first receiver is '" + sReceiverParameters[0] + "', it is of ReceiverType " + sReceiverParameters[2] + " (corresponds to the RCV-file number, whick is same as the caption in the ReceiverType forms)");

				comboBox_Receivers.Items.Clear();
				comboBox_ReceiversAndTypes.Items.Clear();
				foreach (string sReceiver in sReceivers)
				{
					if (sReceiver != "")
					{
						string[] sReceiverParameters = sReceiver.Split('|');
						comboBox_Receivers.Items.Add(sReceiverParameters[0]);
						comboBox_ReceiversAndTypes.Items.Add(sReceiver);
					}
				}

				MessageBox.Show("Found " + (sReceivers.GetLength(0) - 1).ToString() + " receivers");

			}

		}

		private void comboBox_ReceiversAndTypes_SelectedIndexChanged(object sender, EventArgs e)
		{
			string[] sReceiverParameters = comboBox_ReceiversAndTypes.Text.Split('|');
			textBox_ReceiverName.Text = sReceiverParameters[0];
			checkBox_ReceiverActive.Checked = sReceiverParameters[1] == "1";
			textBox_ReceiverType.Text = sReceiverParameters[2];
			textBox_P1.Text = sReceiverParameters[3];
			textBox_P2.Text = sReceiverParameters[4];
			textBox_P3.Text = sReceiverParameters[5];
			textBox_P4.Text = sReceiverParameters[6];
		}

		private void button_UpdateReceiver_Click(object sender, EventArgs e)
		{

			Encoding encoding = Encoding.GetEncoding("Windows-1252");

			string sMessageRow = "";

			if (nimbusSocket.Connected == false)
			{
				return;
			}

			//
			// 200|Berra|sms\gsm modem\cinterion\cinterion bgs2t|0709421013||||
			//
			// or
			//
			// 200|Berra|122|0709421013||||
			//
			// 200                                     -> Update/add receiver
			// Berra                                   -> Receiver Name
			// sms\gsm modem\cinterion\cinterion bgs2t -> Receiver Type as text
			// 122                                     -> Receiver Type as number (same as in Receiver Types form caption and ReceiverType file name (number))
			// 0709421013                              -> Parameter #1 -> Phone number
			//
			// The following empty fields separated by '|' are Parameter #2..#4, their contents depends as #1 of the Receiver Type
			//

			//
			// You need Nimbus version 3 version 3.00.08 or later
			//
			//


			sMessageRow = "200|";
			sMessageRow += textBox_ReceiverName.Text + "|";

			sMessageRow += textBox_ReceiverType.Text + "|";

			sMessageRow += textBox_P1.Text + "|";
			sMessageRow += textBox_P2.Text + "|";
			sMessageRow += textBox_P3.Text + "|";
			sMessageRow += textBox_P4.Text + "|";

			sMessageRow += "\n";

			byte[] byteStream = encoding.GetBytes(sMessageRow);

			nimbusStream.Write(byteStream, 0, byteStream.Length);
			nimbusStream.Flush();


		}

  }
}
